/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  I2C.h - Declaration file for utility functions and global variables
 *  DRV8213_82x4EVM_FIRMWARE
 *  03/15/2023
 *  Pablo Armet
 *  06/15/2023
 *  Murugavel Raju
 *
 ******************************************************************************/

#include "drv82xx/include/drv82xx-RegMap.h"
#include "I2C.h"

int TXData[2];
int RXData;
int TXByteCtr;
int RXByteCtr;
int ls;
char NO_ACK;

//I2C initialization
void i2c_init(void)
{
  UCB1CTL1 |= UCSWRST;                      // Enable SW reset
  UCB1CTL0 = UCMST + UCMODE_3 + UCSYNC;     // I2C Master, synchronous mode
  UCB1CTL1 = UCSSEL_2 + UCSWRST;            // Use SMCLK, keep SW reset
  UCB1BR0 = 200;                            // fSCL = SMCLK/200 = ~100kHz
  //UCB1BR0 = 50;                             // fSCL = SMCLK/50 = ~400kHz
  UCB1BR1 = 0;
  UCB1I2CSA = 0x30;                         // Slave Address
  UCB1CTL1 &= ~UCSWRST;                     // Clear SW reset, resume operation
  UCB1IE = 0;                               // Clear UCB1 interrupts enabled
}

void i2c_write(uint8_t reg_address, uint8_t reg_data)
{
    TXByteCtr = 2;
    RXByteCtr = 0;
    ls = 0;
    TXData[0] = reg_address;
    TXData[1] = reg_data;
    UCA1IE &= ~UCRXIE;                      // Disable USCI_A1 RX interrupt

    while (UCB1CTL1 & UCTXSTP);             // Ensure no ongoing stop condition
    UCB1IE = UCTXIE + UCNACKIE;             // Enable TX interrupt and NACK interrupt
    UCB1CTL1 |= UCTR + UCTXSTT;             // I2C TX, start condition
    __bis_SR_register(LPM0_bits + GIE);     // Enter LPM0
    NO_ACK = 0;                             // Clear NO_ACK indicator
    UCB1IFG = 0;                            // Clear UCB1 interrupt flags
    UCB1IE = 0;                             // Clear UCB1 interrupts enabled
    UCA1IE |= UCRXIE;                       // Enable USCI_A1 RX interrupt
}

uint8_t i2c_read(uint8_t reg_address)
{
    TXByteCtr = 0;
    RXByteCtr = 2;
    TXData[0] = reg_address;
    UCA1IE &= ~UCRXIE;                      // Disable USCI_A1 RX interrupt

    while (UCB1CTL1 & UCTXSTP);             // Ensure no ongoing stop condition
    UCB1IE = UCTXIE + UCNACKIE;             // Enable TX interrupt and NACK interrupt
    UCB1CTL1 |= UCTR + UCTXSTT;             // I2C TX, start condition
    __bis_SR_register(LPM0_bits + GIE);     // Enter LPM0

    if (!NO_ACK)                            // Proceed only if ACK from DRV8214/34
    {
        UCB1IE = 0;                         // Clear UCB1 interrupts enabled
        UCB1CTL1 &= ~UCTR;                  // I2C RX
        UCB1CTL1 |= UCTXSTT;                // Start condition

        while (UCB1CTL1 & UCTXSTT);         // Wait for start condition complete
        UCB1CTL1 |= UCTXSTP;                // Setup stop condition to receive one byte
        while (!(UCB1IFG & UCRXIFG));       // Wait for byte received
        RXData = UCB1RXBUF;                 // Get RX data
        UCA1IE |= UCRXIE;                   // Enable USCI_A1 RX interrupt
        return (RXData);
    }
    else
    {
        NO_ACK = 0;                         // Clear NO_ACK indicator
        UCB1IFG = 0;                        // Clear UCB1 interrupt flags
        UCB1IE = 0;                         // Clear UCB1 interrupts
        UCA1IE |= UCRXIE;                   // Enable USCI_A1 RX interrupt
        return (0);                         // Else return 0
    }
}

//------------------------------------------------------------------------------
// The USCIAB1_ISR is structured such that it can be used to transmit any
// number of bytes by pre-loading TXByteCtr with the byte count.
//------------------------------------------------------------------------------
#pragma vector = USCI_B1_VECTOR
__interrupt void USCI_B1_ISR(void)
{
  switch(__even_in_range(UCB1IV,12))
  {
  case  0: break;                           // Vector  0: No interrupts
  case  2: break;                           // Vector  2: ALIFG
  case  4:                                  // Vector  4: NACKIFG
      NO_ACK = 1;                           // NACK from DRV8214/34
      UCB1CTL1 |= UCTXSTP;                  // I2C stop condition
      UCB1IFG &= ~UCTXIFG;                  // Clear USCI_B1 TX int flag
      __bic_SR_register_on_exit(LPM0_bits); // Exit LPM0
      break;
  case  6: break;                           // Vector  6: STTIFG
  case  8: break;                           // Vector  8: STPIFG
  case 10: break;                           // Vector 10: RXIFG
  case 12:                                  // Vector 12: TXIFG  
      if (TXByteCtr)                        // Check TX byte counter for write CMD
      {
        UCB1TXBUF = TXData[ls];             // Load TX buffer
        TXByteCtr--;                        // Decrement TX byte counter
        ls++;                               // Increment ls variable
      }
      else if (!TXByteCtr && !RXByteCtr)    // Bytes TX completed in i2c write function
      {
          UCB1CTL1 |= UCTXSTP;              // I2C stop condition
          UCB1IFG &= ~UCTXIFG;              // Clear USCI_B1 TX int flag
          __bic_SR_register_on_exit(LPM0_bits); // Exit LPM0
      }

      if (RXByteCtr)                        // Check RX byte counter for read CMD
      {
        if (RXByteCtr == 2)
        {
            UCB1TXBUF = TXData[0];          // Load TX buffer with Register address
            RXByteCtr--;                    // Decrement RX byte counter
        }
        else if (RXByteCtr == 1)            // Address transmitted send a stop condition
        {
            RXByteCtr--;                    // Decrement RX byte counter
            UCB1IFG &= ~UCTXIFG;            // Clear USCI_B1 TX int flag
            __bic_SR_register_on_exit(LPM0_bits); // Exit LPM0
        }
      }

      break;
  default: break;
  }
}


